@@ -216,9 +216,9 @@ namespace KWin
 void Workspace::setActiveWindow(Window *window)
 {
     if (window) {
-        qCDebug(KWIN_CORE) <<"Workspace::setActiveWindow:"<< window->resourceName();
+        qCDebug(KWIN_CORE) << "[Workspace]Set active window:" << window->resourceName();
     } else {
-        qCDebug(KWIN_CORE) <<"Workspace::setActiveWindow: get 0 window";
+        qCDebug(KWIN_CORE) << "[Workspace]Set active window, but window is NULL";
     }
 
     if (m_activeWindow == window) {
@@ -410,18 +410,18 @@ bool Workspace::takeActivity(Window *window, ActivityFlags flags)
         flags &= ~ActivityFocus;
     }
     if (!window->isShown()) { // shouldn't happen, call activateWindow() if needed
-        qCWarning(KWIN_CORE) << "takeActivity: not shown" << window->resourceName();
+        qCWarning(KWIN_CORE) << "[Workspace]Take activity: not shown" << window->resourceName();
         return false;
     }
 
     bool ret = true;
 
     if (flags & ActivityFocus) {
-        qCDebug(KWIN_CORE) << "Workspace::takeActivity: flags & ActivityFocus"  << window->resourceName();
+        qCDebug(KWIN_CORE) << "[Workspace]Take activity: flags & ActivityFocus" << window->resourceName();
         ret &= window->takeFocus();
     }
     if (flags & ActivityRaise) {
-        qCDebug(KWIN_CORE) << "Workspace::takeActivity: flags & ActivityRaise"  << window->resourceName();
+        qCDebug(KWIN_CORE) << "[Workspace]Take activity: flags & ActivityRaise" << window->resourceName();
         workspace()->raiseWindow(window);
     }
 
@@ -511,7 +511,9 @@ bool Workspace::activateNextWindow(Window *window)
         }
     }
 
-    if (!focusCandidate) { // no suitable window under the mouse -> find sth. else
+    // no suitable window under the mouse -> find sth. else
+    bool noFocusFallbackToParent = (qEnvironmentVariableIntValue("KWIN_NO_FOCUS_FALLBACK_TO_PARENT") == 1);
+    if (!focusCandidate && !noFocusFallbackToParent) {
         // first try to pass the focus to the (former) active clients leader
         if (window && window->isTransient()) {
             auto leaders = window->mainWindows();
@@ -520,10 +522,11 @@ bool Workspace::activateNextWindow(Window *window)
                 raiseWindow(focusCandidate); // also raise - we don't know where it came from
             }
         }
-        if (!focusCandidate) {
-            // nope, ask the focus chain for the next candidate
-            focusCandidate = m_focusChain->nextForDesktop(window, desktop);
-        }
+    }
+
+    if (!focusCandidate) {
+        // nope, ask the focus chain for the next candidate
+        focusCandidate = m_focusChain->nextForDesktop(window, desktop);
     }
 
     if (focusCandidate == nullptr) { // last chance: focus the desktop
