@@ -62,6 +62,7 @@ DrmGpu::DrmGpu(DrmBackend *backend, const QString &devNode, int fd, dev_t device
     } else {
         m_cursorSize.setHeight(64);
     }
+    m_gpuCursorSize = m_cursorSize;
 
     int ret = drmGetCap(fd, DRM_CAP_TIMESTAMP_MONOTONIC, &capability);
     if (ret == 0 && capability == 1) {
@@ -75,7 +76,7 @@ DrmGpu::DrmGpu(DrmBackend *backend, const QString &devNode, int fd, dev_t device
 
     // find out what driver this kms device is using
     DrmUniquePtr<drmVersion> version(drmGetVersion(fd));
-    qCDebug(KWIN_DRM) << __func__ << "find gpu " << version->name;
+    qCInfo(KWIN_DRM) << "Find gpu:" << version->name;
     m_isHisi = strstr(version->name, "hisi") || strstr(version->name, "huanglong");
     m_isDisplayLink = strstr(version->name, "udl");
     m_isNVidia = strstr(version->name, "nvidia-drm");
@@ -283,7 +284,7 @@ bool DrmGpu::updateOutputs()
         QVector<DrmConnector*> tmpPendingConnectors;
         QStringList enabledConnIdList = enabledConnIds.trimmed().split(",");
         for (auto idStr : enabledConnIdList) {
-            int id = idStr.toInt();
+            uint32_t id = idStr.toUInt();
             for (DrmConnector *con : qAsConst(existing)) {
                 if (con->id() == id) {
                     qCDebug(KWIN_DRM) << "enable DrmConnector id:" << id;
@@ -297,10 +298,10 @@ bool DrmGpu::updateOutputs()
 
     for (auto it = m_connectors.begin(); it != m_connectors.end();) {
         DrmConnector *conn = it->get();
-        qCDebug(KWIN_DRM) << "on GPU" << m_devNode << "DrmConnector modelName:" << conn->modelName()
-                          << "connectorName:" << conn->connectorName() << "Connection:" << conn->isConnected();
         const auto output = findOutput(conn->id());
         const bool stillExists = existing.contains(conn);
+        qCInfo(KWIN_DRM) << "Update outputs: on GPU" << m_devNode << " model:" << conn->modelName()
+                         << "\nname:" << conn->connectorName() << " Connection:" << conn->isConnected() << " StillExists:" << stillExists;
         if (!stillExists || !conn->isConnected()) {
             if (output) {
                 output->setDrmDpmsMode(Output::DpmsMode::Off);
@@ -311,7 +312,8 @@ bool DrmGpu::updateOutputs()
             const auto pipeline = conn->pipeline();
             m_pipelines << pipeline;
             auto output = new DrmOutput(*it);
-            qCDebug(KWIN_DRM) << "New " << (conn->isNonDesktop() ? "non-desktop " : "") << "output on GPU" << m_devNode << output;
+            qCInfo(KWIN_DRM) << "New" << (conn->isNonDesktop() ? "non-desktop " : "")
+                             << "output on GPU:" << m_devNode << " name:" << output->name();
             m_drmOutputs << output;
             addedOutputs << output;
             Q_EMIT outputAdded(output);
@@ -384,6 +386,16 @@ void DrmGpu::removeOutputs()
     }
 }
 
+void DrmGpu::updateCursorSize(const QSize &size)
+{
+    m_cursorSize = m_gpuCursorSize;
+    if (size.width() < m_gpuCursorSize.width())
+        m_cursorSize.setWidth(size.width());
+
+    if (size.height() < m_gpuCursorSize.height())
+        m_cursorSize.setHeight(size.height());
+}
+
 DrmPipeline::Error DrmGpu::checkCrtcAssignment(QVector<DrmConnector *> connectors, const QVector<DrmCrtc *> &crtcs)
 {
     if (connectors.isEmpty() || crtcs.isEmpty()) {
