@@ -169,21 +169,19 @@ bool DrmOutput::setCursor(CursorSource *source)
     const QMatrix4x4 monitorMatrix = logicalToNativeMatrix(rect(), scale(), transform());
     const QSize cursorSize = m_cursor.source->size();
     const QRect cursorRect = QRect(m_cursor.position, cursorSize);
-    const QRect nativeCursorRect = monitorMatrix.mapRect(cursorRect);
-    if (nativeCursorRect.width() <= m_gpu->cursorSize().width() && nativeCursorRect.height() <= m_gpu->cursorSize().height()) {
-        if (auto beginInfo = layer->beginFrame()) {
-            RenderTarget *renderTarget = &beginInfo->renderTarget;
-            renderTarget->setDevicePixelRatio(scale());
+    m_gpu->updateCursorSize(cursorSize);
+    if (auto beginInfo = layer->beginFrame()) {
+        RenderTarget *renderTarget = &beginInfo->renderTarget;
+        renderTarget->setDevicePixelRatio(scale());
 
-            RenderLayer renderLayer(m_renderLoop.get());
-            renderLayer.setDelegate(std::make_unique<SceneDelegate>(Compositor::self()->cursorScene()));
+        RenderLayer renderLayer(m_renderLoop.get());
+        renderLayer.setDelegate(std::make_unique<SceneDelegate>(Compositor::self()->cursorScene()));
 
-            renderLayer.delegate()->prePaint();
-            renderLayer.delegate()->paint(renderTarget, infiniteRegion());
-            renderLayer.delegate()->postPaint();
+        renderLayer.delegate()->prePaint();
+        renderLayer.delegate()->paint(renderTarget, infiniteRegion());
+        renderLayer.delegate()->postPaint();
 
-            rendered = layer->endFrame(infiniteRegion(), infiniteRegion());
-        }
+        rendered = layer->endFrame(infiniteRegion(), infiniteRegion());
     }
     if (!rendered) {
         if (layer->isVisible()) {
@@ -243,10 +241,15 @@ QList<std::shared_ptr<OutputMode>> DrmOutput::getModes() const
 
     QList<std::shared_ptr<OutputMode>> ret;
     ret.reserve(drmModes.count());
-    qCDebug(KWIN_DRM) << "output getModes, name: " << name() << "model: " << model();
+    qCInfo(KWIN_DRM) << "Output get modes, name:" << name() << "modes count: " << drmModes.count();
+    qCDebug(KWIN_DRM) << "DrmOutput resolution:    refreshRate:";
     for (const auto &drmMode : drmModes) {
         ret.append(drmMode);
-        qCDebug(KWIN_DRM) << "drmoutput resolution:" << drmMode->size() << ", refreshRate:" << drmMode->refreshRate();
+        const auto width = drmMode->size().width();
+        const auto height = drmMode->size().height();
+        qCDebug(KWIN_DRM).noquote() << QString("\t %1 \t %2")
+                                           .arg(QStringLiteral("%1x%2").arg(width).arg(height), -10) // 占10位，左对齐
+                                           .arg(drmMode->refreshRate(), -6); // 占6位，左对齐
     }
     return ret;
 }
@@ -290,6 +293,7 @@ bool DrmOutput::setDrmDpmsMode(DpmsMode mode)
     m_pipeline->setActive(active);
     if (DrmPipeline::commitPipelines({m_pipeline}, active ? DrmPipeline::CommitMode::TestAllowModeset : DrmPipeline::CommitMode::CommitModeset) == DrmPipeline::Error::None) {
         m_pipeline->applyPendingChanges();
+        qCDebug(KWIN_DRM) << "On GPU drm commit pipelines success: " << mode;
         updateDpmsMode(mode);
         if (active) {
             m_gpu->platform()->checkOutputsAreOn();
@@ -445,7 +449,10 @@ bool DrmOutput::queueChanges(const std::shared_ptr<OutputChangeSet> &props)
     m_pipeline->setRenderOrientation(outputToPlaneTransform(props->transform.value_or(transform())));
     m_pipeline->setBrightness(props->brightness.value_or(m_pipeline->brightness()));
     m_pipeline->setCTM(props->ctmValue.value_or(m_pipeline->ctmValue()));
-    m_pipeline->setColorCurves(props->colorCurves.value_or(m_pipeline->colorCurves()));
+    // Check if color curves exist and red channel has data
+    // Don't use GPU check here because GPU might be null, and red is empty on non-Huawei devices
+    if (props->colorCurves.has_value() && props->colorCurves->red.count() > 0)
+        m_pipeline->setColorCurves(props->colorCurves.value());
     m_pipeline->setColorMode(props->colorModeValue.value_or(m_pipeline->colorMode()));
     if (!envOnlySoftwareRotations && m_gpu->atomicModeSetting()) {
         m_pipeline->setBufferOrientation(m_pipeline->renderOrientation());
@@ -490,6 +497,7 @@ void DrmOutput::applyQueuedChanges(const std::shared_ptr<OutputChangeSet> &props
     Q_EMIT changed();
 
     if (isEnabled() && dpmsMode() == DpmsMode::On) {
+        qCDebug(KWIN_DRM) << "Turning outputs on, because output got enabled";
         m_gpu->platform()->turnOutputsOn();
     }
 }
