@@ -162,8 +162,6 @@ EffectsHandlerImpl::EffectsHandlerImpl(Compositor *compositor, WorkspaceScene *s
         const int newDesktop = VirtualDesktopManager::self()->current();
         if (old != 0 && newDesktop != old) {
             Q_EMIT desktopChanged(old, newDesktop, window ? window->effectWindow() : nullptr);
-            // TODO: remove in 4.10
-            Q_EMIT desktopChanged(old, newDesktop);
         }
     });
     connect(ws, &Workspace::currentDesktopChanging, this, [this](uint currentDesktop, QPointF offset, KWin::Window *window) {
@@ -306,7 +304,6 @@ void EffectsHandlerImpl::setupWindowConnections(Window *window)
     connect(window, &Window::windowClosed, this, &EffectsHandlerImpl::slotWindowClosed);
     connect(window, static_cast<void (Window::*)(KWin::Window *, MaximizeMode, bool)>(&Window::clientMaximizedStateChanged),
             this, &EffectsHandlerImpl::slotClientMaximized);
-    connect(window, static_cast<void (Window::*)(KWin::Window *, QRectF, QRectF, MaximizeMode)>(&Window::clientMaximizedChanged), this, &EffectsHandlerImpl::slotClientMaximizedChanged);
     connect(window, static_cast<void (Window::*)(KWin::Window *, MaximizeMode)>(&Window::clientMaximizedStateAboutToChange),
             this, [this](KWin::Window *window, MaximizeMode m) {
                 if (EffectWindowImpl *w = window->effectWindow()) {
@@ -511,13 +508,6 @@ void EffectsHandlerImpl::startPaint()
     m_currentPaintScreenIterator = m_activeEffects.constBegin();
 }
 
-void EffectsHandlerImpl::slotClientMaximizedChanged(KWin::Window *window, QRectF oldRect, QRectF newRect, MaximizeMode maxMode)
-{
-    if (EffectWindowImpl *w = window->effectWindow()) {
-        Q_EMIT windowMaximizedChanged(w, oldRect, newRect, maxMode);
-    }
-}
-
 void EffectsHandlerImpl::slotClientMaximized(Window *window, MaximizeMode maxMode, bool animated)
 {
     if (!animated) {
@@ -1553,6 +1543,7 @@ QPainter *EffectsHandlerImpl::scenePainter()
 
 void EffectsHandlerImpl::toggleEffect(const QString &name)
 {
+    qCDebug(KWIN_CORE) << "EffectsHandler::toggleEffect from dbus : " << name;
     if (isEffectLoaded(name)) {
         unloadEffect(name);
     } else {
@@ -1711,6 +1702,7 @@ void EffectsHandlerImpl::reloadEffect(Effect *effect)
         }
     }
     if (!effectName.isNull()) {
+        qCDebug(KWIN_CORE) << "EffectsHandler::reloadEffect : " << effectName;
         unloadEffect(effectName);
         m_effectLoader->loadEffect(effectName);
     }
@@ -2152,7 +2144,7 @@ QRectF EffectsHandlerImpl::getQuickTileGeometry(KWin::EffectWindow *w, int mode,
     return QRectF();
 }
 
-int EffectsHandlerImpl::windowPId(KWin::EffectWindow *w)
+pid_t EffectsHandlerImpl::windowPId(KWin::EffectWindow *w)
 {
     if (auto client = qobject_cast<Window *>(static_cast<EffectWindowImpl *>(w)->window())) {
         return client->pid();
