@@ -287,7 +287,7 @@ public:
 
     QRectF getQuickTileGeometry(KWin::EffectWindow *w, int mode, QPointF pos) override;
 
-    int windowPId(KWin::EffectWindow *w) override;
+    pid_t windowPId(KWin::EffectWindow *w) override;
     bool isWinAllowSplit(KWin::EffectWindow *) override;
     bool isSplitWin(KWin::EffectWindow *) override;
     void updateQuickTileMode(KWin::EffectWindow *, int mode) override;
@@ -322,7 +322,6 @@ protected Q_SLOTS:
     void slotUnmanagedShown(KWin::Window *);
     void slotWindowClosed(KWin::Window *original, KWin::Deleted *d);
     void slotClientMaximized(KWin::Window *window, MaximizeMode maxMode, bool animated = true);
-    void slotClientMaximizedChanged(KWin::Window *window, QRectF oldRect, QRectF newRect, MaximizeMode maxMode);
     void slotOpacityChanged(KWin::Window *window, qreal oldOpacity);
     void slotClientModalityChanged();
     void slotGeometryShapeChanged(KWin::Window *window, const QRectF &old);
