@@ -49,18 +49,7 @@ ScissorWindow::ScissorWindow() : Effect() {
                                                                                QByteArray(),
                                                                                ":/effects/scissor/fillet.frag");
 
-    {
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-        for (int i = 0; i < KWindowSystem::windows().count(); ++i) {
-            if (EffectWindow *win = effects->findWindow(KWindowSystem::windows().at(i)))
-                windowAdded(win);
-        }
-#endif
-        // FIXME(rewine): can't use KWindowSystem::windows in KF6
-
-        connect(effects, &EffectsHandler::windowAdded, this, &ScissorWindow::windowAdded);
-        connect(effects, &EffectsHandler::windowDeleted, this, &ScissorWindow::windowDeleted);
-    }
+    connect(effects, &EffectsHandler::windowDeleted, this, &ScissorWindow::windowDeleted);
 }
 
 ScissorWindow::~ScissorWindow() {
@@ -238,11 +227,6 @@ bool ScissorWindow::supported() {
     return effects->isOpenGLCompositing() && GLFramebuffer::supported();
 }
 
-void ScissorWindow::windowAdded(EffectWindow *w) {
-    if (!w->hasDecoration())
-        return;
-}
-
 void ScissorWindow::windowDeleted(EffectWindow *w) {
     m_clipMaskMap.erase(w);
 }
