@@ -428,7 +428,7 @@ static QSize pickWindowSize(const QImage &image)
     const uint32_t bytesPerPixel = image.depth() >> 3;
     const uint32_t bytesPerLine = image.bytesPerLine();
 
-    if (image.sizeInBytes() <= maximumDataSize) {
+    if (image.sizeInBytes() <= static_cast<qsizetype>(maximumDataSize)) {
         return image.size();
     }
 
@@ -649,9 +649,9 @@ QString ScreenShotDBusInterface1::screenshotForWindowExtend(qulonglong winid, un
     }
 
      QDBusReply<uint> reply = connection().interface()->servicePid(message().service());
-     uint pid = reply.value();
+     pid_t pid = reply.value();
 
-     int nDockPid = 0;
+     pid_t nDockPid = 0;
      EffectWindowList windowsList = effects->stackingOrder();
      for (int i = 0; i < windowsList.count(); i++) {
          EffectWindow *pWindow = windowsList.at(i);
