@@ -7,5 +7,5 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 #include "logging_p.h"
-Q_LOGGING_CATEGORY(LIBKWINEFFECTS, "libkwineffects", QtWarningMsg)
-Q_LOGGING_CATEGORY(LIBKWINGLUTILS, "libkwinglutils", QtWarningMsg)
+Q_LOGGING_CATEGORY(LIBKWINEFFECTS, "kwin_effects", QtWarningMsg)
+Q_LOGGING_CATEGORY(LIBKWINGLUTILS, "kwin_glutils", QtWarningMsg)
