@@ -34,7 +34,6 @@
 #include <KDesktopFile>
 #include <KLocalizedString>
 #include <KShell>
-#include <KSignalHandler>
 
 // Qt
 #include <QCommandLineParser>
@@ -51,8 +50,8 @@
 #include <sched.h>
 #include <sys/resource.h>
 
-#include <iomanip>
 #include <iostream>
+#include <wayland-server-core.h>
 #if HAVE_SYS_PRCTL_H
 #include <sys/prctl.h>
 #endif
@@ -69,18 +68,29 @@ Q_IMPORT_PLUGIN(KWinIdleTimePoller)
 Q_IMPORT_PLUGIN(ScreencastManagerFactory)
 #endif
 
+Q_LOGGING_CATEGORY(KWIN_LWL, "kwin_lwl", QtWarningMsg);
+
+// Remove the timestamp from the libwayland debug log message
+#define TIME_STR_LENGTH 22
+
+void waylandLogHandler(const char *fmt, va_list args)
+{
+    qCInfo(KWIN_LWL).noquote() << "[libwayland]" << QString::vasprintf(fmt, args).trimmed().mid(TIME_STR_LENGTH);
+}
+
+void waylandDebugHandler(const char *fmt, va_list args)
+{
+    qCDebug(KWIN_LWL).noquote() << "[libwayland]" << QString::vasprintf(fmt, args).trimmed().mid(TIME_STR_LENGTH);
+}
+
 namespace KWin
 {
 
-
-
-
 static void unsetDumpable(int sig)
 {
 #if HAVE_PR_SET_DUMPABLE
     prctl(PR_SET_DUMPABLE, 1);
 #endif
-    qDebug() << "raise signal" << sig;
     signal(sig, SIG_IGN);
     raise(sig);
     return;
@@ -184,8 +194,7 @@ void ApplicationWayland::performStartup()
 
 void ApplicationWayland::continueStartupWithScene()
 {
-    QString currentDateTime = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss.zzz");
-    qDebug() << currentDateTime << "startscene";
+    qInfo() << "Start scene";
     disconnect(Compositor::self(), &Compositor::sceneCreated, this, &ApplicationWayland::continueStartupWithScene);
 
     // Note that we start accepting client connections after creating the Workspace.
@@ -194,7 +203,7 @@ void ApplicationWayland::continueStartupWithScene()
     createPlugins();
 
     if (!waylandServer()->start()) {
-        qFatal("Failed to initialze the Wayland server, exiting now");
+        qFatal("Failed to start the Wayland server, exiting now");
     }
 
     if (operationMode() == OperationModeWaylandOnly) {
@@ -235,8 +244,7 @@ void ApplicationWayland::refreshSettings(const KConfigGroup &group, const QByteA
 
 void ApplicationWayland::startSession()
 {
-    QString currentDateTime = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss.zzz");
-    qDebug() << currentDateTime << "startSession";
+    qDebug() << "Start session";
     KSharedConfig::Ptr kwinSettings = kwinApp()->config();
     m_settingsWatcher = KConfigWatcher::create(kwinSettings);
     connect(m_settingsWatcher.data(), &KConfigWatcher::configChanged, this, &ApplicationWayland::refreshSettings);
@@ -287,8 +295,7 @@ void ApplicationWayland::startSession()
                 continue;
             }
             QString program = arguments.takeFirst();
-            currentDateTime = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss.zzz");
-            qDebug() << currentDateTime << "start application" << program;
+            qInfo() << "Start application" << program;
             // note: this will kill the started process when we exit
             // this is going to happen anyway as we are the wayland and X server the app connects to
             QProcess *p = new QProcess(this);
@@ -299,20 +306,8 @@ void ApplicationWayland::startSession()
             p->startDetached();
             p->deleteLater();
         }
-        currentDateTime = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss.zzz");
-        qDebug() << currentDateTime << "start application finished";
+        qDebug() << "Start application finished";
     }
-    //TODO 这里暂时去掉了kwin日志写入到syslog的功能，原因如下：
-    /* kwin的日志改成写入到syslog之后，其实是qt输出的日志现在改到写入到syslog中了，其实kwin启动的时候，
-        会带着启动xwayland、startdde等，这些应用的日志和wayland协议的日志是通过fprintf打印的，它们只
-        会输出到重定向的文件里边，所以最终导致一部分日志会在syslog里边，一部分会在.kwin.log里边，应用的
-        日志和wayland协议的日志有些是没有时间戳的，这样导致日志执行的时间就无法确定，导致问题反而更难分析
-    */
-}
-
-void ApplicationWayland::exitWayland()
-{
-    _exit(0);
 }
 
 XwaylandInterface *ApplicationWayland::xwayland() const
@@ -324,9 +319,19 @@ XwaylandInterface *ApplicationWayland::xwayland() const
 
 int main(int argc, char *argv[])
 {
-    QString currentDateTime = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss.zzz");
-    qDebug() << currentDateTime << "version" << KWIN_DVERSION << "start";
-
+    // libwayland debug information needs to be captured by kwin,
+    // and kwin uses logging rules to decide whether to enable libwayland debug information.
+    KWin::Logger::instance()->installMessageHandler();
+    wl_log_set_handler_server(waylandLogHandler);
+#ifndef BUILD_WITHOUT_V20_PATCH
+    wl_debug_set_handler_server(waylandDebugHandler);
+#endif
+    // Log: program entry
+    QStringList args;
+    for (int i = 0; i < argc; ++i) {
+        args << QString::fromLocal8Bit(argv[i]);
+    }
+    qInfo() << "kwin_wayland started, version:" << KWIN_DVERSION << ", args:" << args;
     KWin::Application::setupMalloc();
     KWin::Application::setupLocalizedString();
     KWin::gainRealTime();
@@ -342,6 +347,9 @@ int main(int argc, char *argv[])
 
     QProcessEnvironment environment = QProcessEnvironment::systemEnvironment();
 
+    // unset WAYLAND_DEBUG for session apps
+    environment.remove("WAYLAND_DEBUG");
+
     // enforce our internal qpa plugin, unfortunately command line switch has precedence
     setenv("QT_QPA_PLATFORM", "wayland-org.kde.kwin.qpa", true);
 
@@ -352,12 +360,7 @@ int main(int argc, char *argv[])
     a.setupTranslator();
     // reset QT_QPA_PLATFORM so we don't propagate it to our children (e.g. apps launched from the overview effect)
     qunsetenv("QT_QPA_PLATFORM");
-
-    KSignalHandler::self()->watchSignal(SIGTERM);
-    KSignalHandler::self()->watchSignal(SIGINT);
-    KSignalHandler::self()->watchSignal(SIGHUP);
-    QObject::connect(KSignalHandler::self(), &KSignalHandler::signalReceived,
-                     &a, &KWin::ApplicationWayland::exitWayland);
+    qInfo() << "QT_QPA_PLATFORM environment variable reset";
 
     signal(SIGABRT, KWin::unsetDumpable);
     signal(SIGSEGV, KWin::unsetDumpable);
@@ -460,7 +463,7 @@ int main(int argc, char *argv[])
     parser.addOption(noGlobalShortcutsOption);
 
     QCommandLineOption noVtSwitchOption(QStringLiteral("no-vt-switch"),
-                                               i18n("Disable virtual terminal switch global shortcuts support."));
+                                        i18n("Disable virtual terminal switch global shortcuts support."));
     parser.addOption(noVtSwitchOption);
 
 #if KWIN_BUILD_ACTIVITIES
@@ -488,6 +491,7 @@ int main(int argc, char *argv[])
 #endif
 
     if (parser.isSet(replaceOption)) {
+        qInfo() << "Set replaceOption, exiting for wrapper restart";
         QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.KWin"), QStringLiteral("/KWin"),
                                                           QStringLiteral("org.kde.KWin"), QStringLiteral("replace"));
         QDBusConnection::sessionBus().call(msg, QDBus::NoBlock);
@@ -495,8 +499,10 @@ int main(int argc, char *argv[])
     }
 
     a.setDisableMultiScreens(parser.isSet(disableMultiScreens));
+    qInfo() << "Set disableMultiScreens:" << parser.isSet(disableMultiScreens);
 
     if (parser.isSet(exitWithSessionOption)) {
+        qInfo() << "Set exitWithSessionOption, session argument:" << parser.value(exitWithSessionOption);
         a.setSessionArgument(parser.value(exitWithSessionOption));
     }
 
@@ -542,17 +548,17 @@ int main(int argc, char *argv[])
     bool ok = false;
     const int width = parser.value(widthOption).toInt(&ok);
     if (!ok) {
-        std::cerr << "FATAL ERROR incorrect value for width" << std::endl;
+        qCritical() << "FATAL ERROR: incorrect value for width";
         return 1;
     }
     const int height = parser.value(heightOption).toInt(&ok);
     if (!ok) {
-        std::cerr << "FATAL ERROR incorrect value for height" << std::endl;
+        qCritical() << "FATAL ERROR: incorrect value for height";
         return 1;
     }
     const qreal scale = parser.value(scaleOption).toDouble(&ok);
     if (!ok || scale <= 0) {
-        std::cerr << "FATAL ERROR incorrect value for scale" << std::endl;
+        qCritical() << "FATAL ERROR: incorrect value for scale";
         return 1;
     }
 
@@ -562,6 +568,7 @@ int main(int argc, char *argv[])
     const int count = parser.value(outputCountOption).toInt(&ok);
     if (ok) {
         outputCount = std::max(1, count);
+        qInfo() << "Output count set to:" << outputCount;
     }
 
     // TODO: create backend without having the server running
@@ -585,6 +592,7 @@ int main(int argc, char *argv[])
 
     const QString socketName = parser.value(waylandSocketOption);
     if (parser.isSet(waylandSocketFdOption)) {
+        qInfo() << "Using wayland-fd option, socketName:" << socketName;
         bool ok;
         int fd = parser.value(waylandSocketFdOption).toInt(&ok);
         if (ok) {
@@ -592,32 +600,34 @@ int main(int argc, char *argv[])
             fcntl(fd, F_SETFD, FD_CLOEXEC);
             server->display()->addSocketFileDescriptor(fd, socketName);
         } else {
-            std::cerr << "FATAL ERROR: could not parse socket FD" << std::endl;
+            qCritical() << "FATAL ERROR: could not parse socket FD";
             return 1;
         }
     } else {
         // socketName empty is fine here, addSocketName will automatically pick one
         if (!server->display()->addSocketName(socketName)) {
-            std::cerr << "FATAL ERROR: could not add wayland socket " << qPrintable(socketName) << std::endl;
+            qCritical() << "FATAL ERROR: could not add wayland socket" << socketName;
             return 1;
         }
     }
 
     if (!server->init(flags)) {
-        std::cerr << "FATAL ERROR: could not create Wayland server" << std::endl;
+        qCritical() << "FATAL ERROR: could not create Wayland server";
         return 1;
     }
 
     switch (backendType) {
     case BackendType::Kms:
+        qInfo() << "Initializing DRM backend";
         a.setSession(KWin::Session::create());
         if (!a.session()) {
-            std::cerr << "FATAl ERROR: could not acquire a session" << std::endl;
+            qCritical() << "FATAL ERROR: could not acquire a session";
             return 1;
         }
         a.setOutputBackend(std::make_unique<KWin::DrmBackend>(a.session()));
         break;
     case BackendType::Virtual: {
+        qInfo() << "Initializing Virtual backend";
         auto outputBackend = std::make_unique<KWin::VirtualBackend>();
         for (int i = 0; i < outputCount; ++i) {
             outputBackend->addOutput(initialWindowSize, outputScale);
@@ -627,6 +637,7 @@ int main(int argc, char *argv[])
         break;
     }
     case BackendType::X11: {
+        qInfo() << "Initializing X11 backend";
         QString display = parser.value(x11DisplayOption);
         if (display.isEmpty()) {
             display = qgetenv("DISPLAY");
@@ -641,6 +652,7 @@ int main(int argc, char *argv[])
         break;
     }
     case BackendType::Wayland: {
+        qInfo() << "Initializing Wayland backend";
         QString socketName = parser.value(waylandDisplayOption);
         if (socketName.isEmpty()) {
             socketName = qgetenv("WAYLAND_DISPLAY");
@@ -664,6 +676,7 @@ int main(int argc, char *argv[])
     a.setProcessStartupEnvironment(environment);
 
     if (parser.isSet(xwaylandOption)) {
+        qInfo() << "Set xwaylandOption, preparing to start Xwayland";
         a.setStartXwayland(true);
 
         if (parser.isSet(xwaylandListenFdOption)) {
@@ -691,6 +704,7 @@ int main(int argc, char *argv[])
 
     a.setApplicationsToStart(parser.positionalArguments());
     a.setInputMethodServerToStart(parser.value(inputMethodOption));
+    qInfo() << "Starting main loop";
     a.start();
 
     return a.exec();
