@@ -246,13 +246,6 @@ bool ScriptedEffect::init(const QString &effectName, const QString &pathToScript
     QQmlEngine::setObjectOwnership(this, QQmlEngine::CppOwnership);
     globalObject.setProperty(QStringLiteral("effect"), selfObject);
 
-    // desktopChanged is overloaded, which is problematic. Old code exposed the signal also
-    // with parameters. QJSEngine does not so we have to fake it.
-    effectsObject.setProperty(QStringLiteral("desktopChanged(int,int)"),
-                              effectsObject.property(QStringLiteral("desktopChangedLegacy")));
-    effectsObject.setProperty(QStringLiteral("desktopChanged(int,int,KWin::EffectWindow*)"),
-                              effectsObject.property(QStringLiteral("desktopChanged")));
-
     globalObject.setProperty(QStringLiteral("Effect"),
                              m_engine->newQMetaObject(&ScriptedEffect::staticMetaObject));
     globalObject.setProperty(QStringLiteral("KWin"),
