@@ -13,7 +13,6 @@
 #include "main.h"
 #include "workspace.h"
 
-#include "qwayland-server-org-kde-kwin-outputdevice.h"
 #include "qwayland-server-outputmanagement.h"
 
 #include <optional>
@@ -23,7 +22,7 @@ using namespace KWin;
 namespace KWaylandServer
 {
 
-static const quint32 s_version = 4;
+static const quint32 s_version = 5;
 
 class OutputManagementInterfacePrivate : public QtWaylandServer::org_kde_kwin_outputmanagement
 {
@@ -32,20 +31,18 @@ public:
 
 protected:
     void org_kde_kwin_outputmanagement_create_configuration(Resource *resource, uint32_t id) override;
+
+private:
+    Display *const m_display;
 };
 
 class OutputConfigurationInterface : public QObject, QtWaylandServer::org_kde_kwin_outputconfiguration
 {
     Q_OBJECT
 public:
-    explicit OutputConfigurationInterface(wl_resource *resource);
+    explicit OutputConfigurationInterface(wl_resource *resource, Display *display = nullptr);
     virtual ~OutputConfigurationInterface();
 
-    bool applied = false;
-    bool invalid = false;
-    OutputConfiguration config;
-    QVector<std::pair<uint32_t, OutputDeviceInterface *>> outputOrder;
-
 protected:
     void org_kde_kwin_outputconfiguration_enable(Resource *resource, wl_resource *outputdevice, int32_t enable) override;
     void org_kde_kwin_outputconfiguration_mode(Resource *resource, wl_resource *outputdevice, int32_t mode_id) override;
@@ -61,46 +58,53 @@ protected:
     void org_kde_kwin_outputconfiguration_set_vrr_policy(Resource *resource, wl_resource *outputdevice, uint32_t policy) override;
     void org_kde_kwin_outputconfiguration_brightness(Resource *resource, wl_resource *outputdevice, int32_t brightness) override;
     void org_kde_kwin_outputconfiguration_ctm(Resource *resource, wl_resource *outputdevice, int32_t red, int32_t green, int32_t blue) override;
+    void org_kde_kwin_outputconfiguration_set_color_mode(Resource *resource, wl_resource *outputdevice, uint32_t color_mode) override;
+
+private:
+    bool applied = false;
+    bool invalid = false;
+    OutputConfiguration config;
+    QVector<std::pair<uint32_t, OutputDeviceInterface *>> outputOrder;
+    Display *const m_display = nullptr;
 };
 
 OutputManagementInterfacePrivate::OutputManagementInterfacePrivate(Display *display)
     : QtWaylandServer::org_kde_kwin_outputmanagement(*display, s_version)
+    , m_display(display)
 {
 }
 
 void OutputManagementInterfacePrivate::org_kde_kwin_outputmanagement_create_configuration(Resource *resource, uint32_t id)
 {
-    qCDebug(KWIN_CORE) << "outputv1:resource " << resource << " id " << id;
     wl_resource *config_resource = wl_resource_create(resource->client(), &org_kde_kwin_outputconfiguration_interface, resource->version(), id);
     if (!config_resource) {
-        qCDebug(KWIN_CORE) << "outputv1:resource " << resource << " wl_client_post_no_memory";
+        qCWarning(KWIN_CORE) << "Outputv1->create configuration failed!";
         wl_client_post_no_memory(resource->client());
         return;
     }
-    new OutputConfigurationInterface(config_resource);
+    new OutputConfigurationInterface(config_resource, m_display);
 }
 
 OutputManagementInterface::OutputManagementInterface(Display *display, QObject *parent)
     : QObject(parent)
     , d(new OutputManagementInterfacePrivate(display))
 {
-    qCDebug(KWIN_CORE) << "outputv1:" << this << " construct configurationM ";
+    qCDebug(KWIN_CORE) << "Outputv1->construct menagement interface";
 }
 
 OutputManagementInterface::~OutputManagementInterface()
 {
-    qCDebug(KWIN_CORE) << "outputv1:" << this << " destruct configurationM ";
 }
 
-OutputConfigurationInterface::OutputConfigurationInterface(wl_resource *resource)
+OutputConfigurationInterface::OutputConfigurationInterface(wl_resource *resource, Display *display)
     : QtWaylandServer::org_kde_kwin_outputconfiguration(resource)
+    , m_display(display)
 {
-    qCDebug(KWIN_CORE) << "outputv1:" << this << " construct configuration ";
+    qCDebug(KWIN_CORE) << "Outputv1->construct configuration interface";
 }
 
 OutputConfigurationInterface::~OutputConfigurationInterface()
 {
-    qCDebug(KWIN_CORE) << "outputv1:" << this << " destruct configuration ";
 }
 
 void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_enable(Resource *resource, wl_resource *outputdevice, int32_t enable)
@@ -111,8 +115,6 @@ void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_enable(Resou
         return;
     }
 
-    qCDebug(KWIN_CORE) << "outputv1:" << this << " outputdevice " << output << " invalid " << invalid << " enable " << enable;
-
     config.changeSet(output->handle())->enabled = enable;
 }
 
@@ -124,14 +126,8 @@ void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_mode(Resourc
         return;
     }
 
-    qCDebug(KWIN_CORE) << "outputv1:" << this << " outputdevice " << output << " invalid " << invalid << " mode_id " << mode_id;
-
     OutputDeviceModeInterface *mode = output->getMode(mode_id);
     if (output && mode) {
-        qCDebug(KWIN_CORE) << "outputv1:" << this << " outputdevice " << output
-                << " invalid " << invalid << " mode_id " << mode_id
-                << " size " << mode->size() << " refreshRate "
-                << mode->refreshRate() << " mode " << mode->flags();
         config.changeSet(output->handle())->mode = mode->m_handle.lock();
     } else {
         invalid = true;
@@ -146,8 +142,6 @@ void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_transform(Re
         return;
     }
 
-    qCDebug(KWIN_CORE) << "outputv1:" << this << " outputdevice " << output << " invalid " << invalid << " transform " << transform;
-
     auto toTransform = [transform]() {
         switch (transform) {
         case WL_OUTPUT_TRANSFORM_90:
@@ -181,8 +175,6 @@ void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_position(Res
         return;
     }
 
-    qCDebug(KWIN_CORE) << "outputv1:" << this << " outputdevice " << output << " invalid " << invalid << " x " << x << " y " << y;
-
     config.changeSet(output->handle())->pos = QPoint(x, y);
 }
 
@@ -194,8 +186,6 @@ void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_scale(Resour
         return;
     }
 
-    qCDebug(KWIN_CORE) << "outputv1:" << this << " outputdevice " << output << " invalid " << invalid << " scale " << scale;
-
     qreal doubleScale = wl_fixed_to_double(scale);
 
     // the fractional scaling protocol only speaks in unit of 120ths
@@ -213,14 +203,16 @@ void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_scale(Resour
 
 void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_scalef(Resource *resource, wl_resource *outputdevice, wl_fixed_t scale)
 {
+    if (resource->version() < ORG_KDE_KWIN_OUTPUTCONFIGURATION_SCALEF_SINCE_VERSION) {
+        return;
+    }
+
     OutputDeviceInterface *output = OutputDeviceInterface::get(outputdevice);
     if (!output || output->invalid()) {
         invalid = true;
         return;
     }
 
-    qCDebug(KWIN_CORE) << "outputv1:" << this << " outputdevice " << output << " invalid " << invalid << " scale " << scale;
-
     qreal doubleScale = wl_fixed_to_double(scale);
 
     // the fractional scaling protocol only speaks in unit of 120ths
@@ -238,14 +230,16 @@ void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_scalef(Resou
 
 void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_overscan(Resource *resource, wl_resource *outputdevice, uint32_t overscan)
 {
+    if (resource->version() < ORG_KDE_KWIN_OUTPUTCONFIGURATION_OVERSCAN_SINCE_VERSION) {
+        return;
+    }
+
     OutputDeviceInterface *output = OutputDeviceInterface::get(outputdevice);
     if (!output || output->invalid()) {
         invalid = true;
         return;
     }
 
-    qCDebug(KWIN_CORE) << "outputv1:" << this << " outputdevice " << output << " invalid " << invalid << " overscan " << overscan;
-
     if (overscan > 100) {
         qCWarning(KWIN_CORE) << "Invalid overscan requested:" << overscan;
         return;
@@ -255,14 +249,16 @@ void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_overscan(Res
 
 void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_set_vrr_policy(Resource *resource, wl_resource *outputdevice, uint32_t policy)
 {
+    if (resource->version() < ORG_KDE_KWIN_OUTPUTCONFIGURATION_SET_VRR_POLICY_SINCE_VERSION) {
+        return;
+    }
+
     OutputDeviceInterface *output = OutputDeviceInterface::get(outputdevice);
     if (!output || output->invalid()) {
         invalid = true;
         return;
     }
 
-    qCDebug(KWIN_CORE) << "outputv1:" << this << " outputdevice " << output << " invalid " << invalid << " policy " << policy;
-
     if (policy > static_cast<uint32_t>(RenderLoop::VrrPolicy::Automatic)) {
         qCWarning(KWIN_CORE) << "Invalid Vrr Policy requested:" << policy;
         return;
@@ -272,17 +268,18 @@ void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_set_vrr_poli
 
 void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_colorcurves(Resource *resource, wl_resource *outputdevice, wl_array *red, wl_array *green, wl_array *blue)
 {
+    if (resource->version() < ORG_KDE_KWIN_OUTPUTCONFIGURATION_COLORCURVES_SINCE_VERSION) {
+        return;
+    }
+
     OutputDeviceInterface *output = OutputDeviceInterface::get(outputdevice);
     if (!output || output->invalid()) {
         invalid = true;
         return;
     }
-    qCDebug(KWIN_CORE) << "outputv1:" << this << " outputdevice " << output
-            << " invalid " << invalid << " red " << red << " green " << green << " blue " << blue;
-
     Output::ColorCurves oldCc = output->handle()->colorCurves();
 
-    auto checkArg = [](const wl_array *newColor, const QVector<quint16> &oldColor) {
+    static auto checkArg = [](const wl_array *newColor, const QVector<quint16> &oldColor) {
         return (newColor->size % sizeof(uint16_t) == 0) && (newColor->size / sizeof(uint16_t) == static_cast<size_t>(oldColor.size()));
     };
     if (!checkArg(red, oldCc.red) || !checkArg(green, oldCc.green) || !checkArg(blue, oldCc.blue)) {
@@ -292,7 +289,7 @@ void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_colorcurves(
 
     Output::ColorCurves cc;
 
-    auto fillVector = [](const wl_array *array, QVector<quint16> *v) {
+    static auto fillVector = [](const wl_array *array, QVector<quint16> *v) {
         const uint16_t *pos = (uint16_t *)array->data;
 
         while ((char *)pos < (char *)array->data + array->size) {
@@ -309,47 +306,66 @@ void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_colorcurves(
 
 void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_brightness(Resource *resource, wl_resource *outputdevice, int32_t brightness)
 {
+    if (resource->version() < ORG_KDE_KWIN_OUTPUTCONFIGURATION_BRIGHTNESS_SINCE_VERSION) {
+        return;
+    }
+
     OutputDeviceInterface *output = OutputDeviceInterface::get(outputdevice);
     if (!output || output->invalid()) {
         invalid = true;
         return;
     }
-
-    qCDebug(KWIN_CORE) << "outputv1:" << this << " outputdevice " << output << " invalid " << invalid << " brightness " << brightness;
-
     config.changeSet(output->handle())->brightness = brightness;
 }
 
 void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_ctm(Resource *resource, wl_resource *outputdevice, int32_t r, int32_t g, int32_t b)
 {
+    if (resource->version() < ORG_KDE_KWIN_OUTPUTCONFIGURATION_CTM_SINCE_VERSION) {
+        return;
+    }
+
+    OutputDeviceInterface *output = OutputDeviceInterface::get(outputdevice);
+    if (!output || output->invalid()) {
+        invalid = true;
+        return;
+    }
+    config.changeSet(output->handle())->ctmValue = Output::CtmValue{(uint16_t)r, (uint16_t)g, (uint16_t)b};
+}
+
+void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_set_color_mode(Resource *resource, wl_resource *outputdevice, uint32_t color_mode)
+{
+    if (resource->version() < ORG_KDE_KWIN_OUTPUTCONFIGURATION_SET_COLOR_MODE_SINCE_VERSION) {
+        return;
+    }
+
     OutputDeviceInterface *output = OutputDeviceInterface::get(outputdevice);
     if (!output || output->invalid()) {
         invalid = true;
         return;
     }
 
-    qCDebug(KWIN_CORE) << "outputv1:" << this << " outputdevice " << output << " invalid " << invalid << " r " << r << " g " << g << " b " << b;
-
-    config.changeSet(output->handle())->ctmValue = Output::CtmValue{(uint16_t)r, (uint16_t)g, (uint16_t)b};
+    const auto mode = static_cast<Output::ColorMode>(color_mode);
+    config.changeSet(output->handle())->colorModeValue = mode;
 }
 
 void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_destroy(Resource *resource)
 {
-    qCDebug(KWIN_CORE) << "outputv1:" << this << " resource " << resource << " invalid " << invalid;
+    const auto connection = m_display->getConnection(resource->client());
+    qCDebug(KWIN_CORE) << "Outputv1->destroy configuration:" << connection->executableComm() << " pid:" << connection->processId();
     wl_resource_destroy(resource->handle);
 }
 
 void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_destroy_resource(Resource *resource)
 {
-    qCDebug(KWIN_CORE) << "outputv1:" << this << " resource " << resource << " invalid " << invalid;
+    const auto connection = m_display->getConnection(resource->client());
+    qCDebug(KWIN_CORE) << "Outputv1->destroy resource:" << connection->executableComm() << " pid:" << connection->processId();
     delete this;
 }
 
 void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_apply(Resource *resource)
 {
-    qCDebug(KWIN_CORE) << "outputv1:" << this << " resource " << resource << " invalid " << invalid;
     if (applied) {
-        qCWarning(KWIN_CORE) << "Rejecting because an output configuration can be applied only once";
+        qCWarning(KWIN_CORE) << "Rejecting configuration change because an output configuration can be applied only once";
         return;
     }
 
@@ -363,9 +379,14 @@ void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_apply(Resour
 
     const auto allOutputs = kwinApp()->outputBackend()->outputs();
     const bool allDisabled = !std::any_of(allOutputs.begin(), allOutputs.end(), [this](const auto &output) {
-        return config.constChangeSet(output)->enabled;
+        const auto changeset = config.constChangeSet(output);
+        if (changeset && changeset->enabled.has_value()) {
+            return *changeset->enabled;
+        } else {
+            return output->isEnabled();
+        }
     });
-    if (allDisabled) {
+    if (allDisabled && !qEnvironmentVariableIsSet("KWIN_ALLOW_DISABLE_ALL_OUTPUTS")) {
         qCWarning(KWIN_CORE) << "Disabling all outputs through configuration changes is not allowed";
         applied = false;
         send_failed();
@@ -375,7 +396,6 @@ void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_apply(Resour
     QVector<Output *> sortedOrder;
     if (!outputOrder.empty()) {
         const int desktopOutputs = std::count_if(allOutputs.begin(), allOutputs.end(), [](Output *output) {
-            qCDebug(KWIN_CORE) << "outputv1:resource " << " isNonDesktop ";
             return !output->isNonDesktop();
         });
         if (outputOrder.size() != desktopOutputs) {
@@ -385,11 +405,16 @@ void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_apply(Resour
             return;
         }
         outputOrder.erase(std::remove_if(outputOrder.begin(), outputOrder.end(), [this](const auto &pair) {
-                              return !config.constChangeSet(pair.second->handle())->enabled;
+                            const auto changeset = config.constChangeSet(pair.second->handle());
+                              if (changeset && changeset->enabled.has_value()) {
+                                  return !changeset->enabled.value();
+                              } else {
+                                  return !pair.second->handle()->isEnabled();
+                              }
                           }),
                           outputOrder.end());
         std::sort(outputOrder.begin(), outputOrder.end(), [](const auto &pair1, const auto &pair2) {
-            qCDebug(KWIN_CORE) << "outputv1:resource " << " pair2 ";
+            qCDebug(KWIN_CORE) << "Outputv1->pair output order: pair1:" << pair1.first << " pair2:" << pair2.first;
             return pair1.first < pair2.first;
         });
         uint32_t i = 1;
@@ -404,12 +429,12 @@ void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_apply(Resour
         }
         sortedOrder.reserve(outputOrder.size());
         std::transform(outputOrder.begin(), outputOrder.end(), std::back_inserter(sortedOrder), [](const auto &pair) {
-            qCDebug(KWIN_CORE) << "outputv1:resource " << " second handle";
             return pair.second->handle();
         });
     }
     if (workspace()->applyOutputConfiguration(config, sortedOrder)) {
-        qCDebug(KWIN_CORE) << "outputv1:" << this << " resource " << resource << " send_applied";
+        const auto connection = m_display->getConnection(resource->client());
+        qCDebug(KWIN_CORE) << "Outputv1->send applied: command:" << connection->executableComm() << " pid:" << connection->processId();
         applied = false;
         send_applied();
     } else {
@@ -419,7 +444,6 @@ void OutputConfigurationInterface::org_kde_kwin_outputconfiguration_apply(Resour
     }
     config.reset();
 }
-
 }
 
-#include "outputmanagement_v1_interface.moc"
\ No newline at end of file
+#include "outputmanagement_v1_interface.moc"
