@@ -228,7 +228,6 @@ void SeatInterfacePrivate::registerDataControlDevice(DataControlDeviceV1Interfac
         }
         q->setPrimarySelection(dataDevice->primarySelection());
     });
-    qCWarning(KWIN_CORE) << "new dataDevice " << dataDevice << "@" << dataDevice->processId();
     if (currentSelection) {
         verifySelection(dataDevice, currentSelection);
     }
@@ -346,7 +345,8 @@ bool SeatInterfacePrivate::skipVerify(pid_t pid)
     sprintf(proc_pid_path, "/proc/%d/status", pid);
     FILE *fp = fopen(proc_pid_path, "r");
     if (NULL != fp) {
-        fgets(buf, BUF_SIZE-1, fp);
+        char* fbuffer = fgets(buf, BUF_SIZE-1, fp);
+        Q_UNUSED(fbuffer);
         fclose(fp);
         sscanf(buf, "%*s %s", task_name);
     }
@@ -360,16 +360,8 @@ bool SeatInterfacePrivate::verifySelection(AbstractDataDevice *dataDevice, Abstr
         return false;
     }
 
-    QVector<pid_t> allDataControlDevices;
-    for (auto control : std::as_const(dataControlDevices)) {
-        allDataControlDevices.append(control->processId());
-    }
-
-    for (auto device : std::as_const(dataDevices)) {
-        allDataControlDevices.append(device->processId());
-    }
     if (dataDevice->deviceType() == AbstractDataDevice::DeviceType::DeviceType_DataControl) {
-        if (allDataControlDevices.contains(dataDevice->processId())) {
+        if (skipVerify(dataDevice->processId())) {
             if (dataSource->extSourceType() == AbstractDataSource::SourceType::FromPrimary) {
                 dataDevice->sendPrimarySelection(dataSource);
             } else {
@@ -396,13 +388,7 @@ bool SeatInterfacePrivate::verifySelection(AbstractDataDevice *dataDevice, Abstr
             dataSource->extSourceType() == AbstractDataSource::SourceType::FromPrimary) {
         dataDevice->sendPrimarySelection(dataSource);
     } else {
-        // The client is updated only after the signal sent by itself or dataControlDevices
-        if (dataSource->extSourceType() == AbstractDataSource::SourceType::FromPrimary) {
-                dataDevice->sendPrimarySelection(dataSource);
-            } else if (dataDevice->processId() == dataSource->pid
-            || allDataControlDevices.contains(dataDevice->processId()) || allDataControlDevices.contains(dataSource->pid)) {
-            dataDevice->sendSelection(dataSource);
-        }
+        dataDevice->sendSelection(dataSource);
     }
     return false;
 }
@@ -454,19 +440,11 @@ void SeatInterfacePrivate::handleCopySecurityVerified(uint32_t serial, uint32_t
             verifyingState.remove(serial);
             return;
         }
-        QVector<pid_t> allDataControlDevices;
-        for (auto control : std::as_const(dataControlDevices)) {
-            allDataControlDevices.append(control->processId());
-        }
-
-        for (auto device : std::as_const(dataDevices)) {
-            allDataControlDevices.append(device->processId());
-        }
 
         switch (state->deviceType) {
         case AbstractDataDevice::DeviceType::DeviceType_Data:
-            if (state->dataDevice->processId() == state->dataSource->pid 
-                || allDataControlDevices.contains(state->dataDevice->processId()) || allDataControlDevices.contains(state->dataSource->pid)) {
+            if (globalKeyboard.focus.selections.contains(static_cast<DataDeviceInterface*>(state->dataDevice)) ||
+                state->dataDevice->processId() == state->dataSource->pid) {
                 state->dataDevice->sendSelection(state->dataSource);
             }
             break;
@@ -499,9 +477,7 @@ void SeatInterfacePrivate::handleCopySecurityVerified(uint32_t serial, uint32_t
                 Q_EMIT q->primarySelectionChanged(state->dataSource);
             } else {
                 currentSelection = currentVerifySelection;
-                if (allDataControlDevices.contains(state->dataSource->pid)){
-                    Q_EMIT q->selectionChanged(state->dataSource);
-                }
+                Q_EMIT q->selectionChanged(state->dataSource);
             }
         }
     }
@@ -946,17 +922,34 @@ void SeatInterface::notifyPointerAxisToClient(Qt::Orientation orientation, qint3
     QPointer<SurfaceInterface> sur = d->pointer->focusedSurface();
     if (sur != surface) {
         d->pointer->sendEnter(surface, pos, serial);
+    } else {
+        stopPendingFocusedSurfaceRecovery();
     }
-    d->pointer->sendAxis(orientation, delta, 0, PointerAxisSource::Wheel);
-    d->pointer->sendFrame();
-    static QTimer timer;
-    timer.setSingleShot(true);
-    timer.start(300);
-    static QMetaObject::Connection c;
-    disconnect(c);
-    c = connect(&timer, &QTimer::timeout, this, [this, sur, pos, serial] {
-        if (sur != nullptr && d->pointer != nullptr) {
-            d->pointer->sendEnter(sur, pos, serial);
+    bool ok = false;
+    int delay = qEnvironmentVariableIntValue("KWIN_SEAT_NOTIFY_AXIS_DELAY", &ok);
+    delay = ok && delay > 0 ? delay : 150; // default 150ms
+    QTimer::singleShot(delay, this, [this, sur, pos, serial, surface, orientation, delta, delay] {
+        stopPendingFocusedSurfaceRecovery();
+        if (surface != d->pointer->focusedSurface()) {
+            qCWarning(KWIN_CORE) << "Surface changed during axis notification.";
+            return;
+        }
+        d->pointer->sendAxis(orientation, delta, 0, PointerAxisSource::Wheel);
+        d->pointer->sendFrame();
+        d->notifyTimer.setSingleShot(true);
+        d->notifyTimer.setInterval(delay);
+        Q_ASSERT(!d->notifyTimeoutConn);
+        d->notifyTimeoutConn = connect(&d->notifyTimer, &QTimer::timeout, this, [this, sur, pos, serial, surface] {
+            stopPendingFocusedSurfaceRecovery();
+            Q_ASSERT(surface == d->pointer->focusedSurface());
+            if (sur != nullptr && d->pointer != nullptr) {
+                d->pointer->sendEnter(sur, pos, serial);
+            }
+        });
+        if (d->notifyTimeoutConn) {
+            d->notifyTimer.start();
+        } else {
+            qCWarning(KWIN_CORE) << "Failed to post recovering focused surface" << sur << "due to connection error.";
         }
     });
 }
@@ -1592,17 +1585,7 @@ void SeatInterface::setSelection(AbstractDataSource *selection, bool sendControl
 
     if (!doVerify) {
         d->currentSelection = selection;
-        // xwayland client is updated only when dataControlDevices signal or selection is empty
-        if (!selection) {
-            Q_EMIT selectionChanged(selection);
-        } else {
-            for (auto control : std::as_const(d->dataControlDevices)) {
-                if (control->processId() == selection->pid){
-                    Q_EMIT selectionChanged(selection);
-                    break;
-                }
-            }
-        }
+        Q_EMIT selectionChanged(selection);
     }
 }
 
@@ -1696,6 +1679,14 @@ void SeatInterface::addSecurityInterface(DDESecurityInterface* security)
     d->addSecurityInterface(security);
 }
 
+void SeatInterface::stopPendingFocusedSurfaceRecovery()
+{
+    if (d->notifyTimer.isActive())
+        d->notifyTimer.stop();
+    if (d->notifyTimeoutConn)
+        disconnect(d->notifyTimeoutConn);
+}
+
 uint32_t SeatInterface::verifySelectionForX11(AbstractDataSource *source, int target)
 {
     return d->verifySelectionForX11(source, target);
