@@ -20,6 +20,7 @@
 #include "surface_interface_p.h"
 #include "surfacerole_p.h"
 #include "utils.h"
+#include "viewporter_interface_p.h"
 
 #include <wayland-server.h>
 // std
@@ -271,6 +272,14 @@ void SurfaceInterfacePrivate::surface_attach(Resource *resource, struct ::wl_res
         return;
     }
     pending.buffer = compositor->display()->clientBufferForResource(buffer);
+    if (pending.buffer) {
+        // Client might reuse a skipped buffer that has been released.
+        // Attaching released buffer means it may need a release again
+        // (If it's committed, thus referenced.), so we should clear
+        // its released state to emit a new buffer release when its
+        // reference count reaches 0.
+        pending.buffer->resetSkipReleasedState();
+    }
 }
 
 void SurfaceInterfacePrivate::surface_damage(Resource *, int32_t x, int32_t y, int32_t width, int32_t height)
@@ -431,17 +440,17 @@ QMatrix4x4 SurfaceInterfacePrivate::buildSurfaceToBufferMatrix()
         break;
     case KWin::Output::Transform::Rotated90:
     case KWin::Output::Transform::Flipped90:
-        surfaceToBufferMatrix.translate(0, bufferSize.height() / current.bufferScale);
+        surfaceToBufferMatrix.translate(0, qreal(bufferSize.height()) / current.bufferScale);
         surfaceToBufferMatrix.rotate(-90, 0, 0, 1);
         break;
     case KWin::Output::Transform::Rotated180:
     case KWin::Output::Transform::Flipped180:
-        surfaceToBufferMatrix.translate(bufferSize.width() / current.bufferScale, bufferSize.height() / current.bufferScale);
+        surfaceToBufferMatrix.translate(qreal(bufferSize.width()) / current.bufferScale, qreal(bufferSize.height()) / current.bufferScale);
         surfaceToBufferMatrix.rotate(-180, 0, 0, 1);
         break;
     case KWin::Output::Transform::Rotated270:
     case KWin::Output::Transform::Flipped270:
-        surfaceToBufferMatrix.translate(bufferSize.width() / current.bufferScale, 0);
+        surfaceToBufferMatrix.translate(qreal(bufferSize.width()) / current.bufferScale, 0);
         surfaceToBufferMatrix.rotate(-270, 0, 0, 1);
         break;
     }
@@ -449,12 +458,12 @@ QMatrix4x4 SurfaceInterfacePrivate::buildSurfaceToBufferMatrix()
     switch (current.bufferTransform) {
     case KWin::Output::Transform::Flipped:
     case KWin::Output::Transform::Flipped180:
-        surfaceToBufferMatrix.translate(bufferSize.width() / current.bufferScale, 0);
+        surfaceToBufferMatrix.translate(qreal(bufferSize.width()) / current.bufferScale, 0);
         surfaceToBufferMatrix.scale(-1, 1);
         break;
     case KWin::Output::Transform::Flipped90:
     case KWin::Output::Transform::Flipped270:
-        surfaceToBufferMatrix.translate(bufferSize.height() / current.bufferScale, 0);
+        surfaceToBufferMatrix.translate(qreal(bufferSize.height()) / current.bufferScale, 0);
         surfaceToBufferMatrix.scale(-1, 1);
         break;
     default:
@@ -588,7 +597,7 @@ void SurfaceInterfacePrivate::applyState(SurfaceState *next)
         if (bufferRef) {
             bufferRef->ref();
             if (isSkipBuffer) {
-                bufferRef->sendRelease();
+                bufferRef->skipRelease();
             }
         }
     }
@@ -1135,4 +1144,9 @@ void SurfaceInterface::skipBuffer()
     d->isSkipBuffer = true;
 }
 
+ViewportInterface *SurfaceInterface::viewportExtension() const
+{
+    return d->viewportExtension;
+}
+
 } // namespace KWaylandServer
