@@ -122,7 +122,7 @@ public:
     {
         QString enableBlackList = QString::fromUtf8(qgetenv("KWIN_ENABLE_INTERFACES_BLACKLIST"));
         if (enableBlackList.isEmpty() || enableBlackList.toLower() == "false") {
-            qCDebug(KWIN_CORE) << "Interfaces blacklist is not enabled";
+            qCInfo(KWIN_CORE) << "Interfaces blacklist is not enabled";
             m_enableBlackList = false;
         }
     }
@@ -195,7 +195,7 @@ public:
         }
 
         if (client->executablePath().isEmpty()) {
-            qCDebug(KWIN_CORE) << "Could not identify process with pid" << client->processId();
+            qCWarning(KWIN_CORE) << "Client executable path is empty, pid:" << client->processId();
             return false;
         }
 
@@ -229,7 +229,7 @@ public:
                 return false;
             }
         }
-        qCDebug(KWIN_CORE) << "authorized" << client->executablePath() << interfaceName;
+        qCDebug(KWIN_CORE) << "Interface authorized, client:" << client->executablePath() << " interface:" << interfaceName;
         return true;
     }
 };
@@ -257,10 +257,13 @@ KWaylandServer::ClientConnection *WaylandServer::inputMethodConnection() const
 
 void WaylandServer::registerWindow(Window *window)
 {
+    qCInfo(KWIN_CORE) << "Registering window, window name:" << window->resourceName() << ", window id:" << window->window();
     if (window->readyForPainting()) {
         Q_EMIT windowAdded(window);
+        qCInfo(KWIN_CORE) << "Window ready for painting, added. Window id:" << window->window();
     } else {
         connect(window, &Window::windowShown, this, &WaylandServer::windowShown);
+        qCInfo(KWIN_CORE) << "Window not ready for painting, waiting for windowShown signal. Window id:" << window->window();
     }
     m_windows << window;
 }
@@ -335,7 +338,6 @@ void WaylandServer::registerXdgToplevelV6Window(XdgToplevelV6Window *window)
 
 void WaylandServer::registerXdgGenericWindow(Window *window)
 {
-    FUNC_DEBUG_LOG(Q_FUNC_INFO, window->surface()->id());
     if (auto toplevel = qobject_cast<XdgToplevelWindow *>(window)) {
         registerXdgToplevelWindow(toplevel);
         return;
@@ -610,6 +612,20 @@ bool WaylandServer::init(InitializationFlags flags)
         [this] (XWaylandKeyboardGrabV1Interface *grab) {
             m_grabClient = grab;
             qDebug() << "grab successfully";
+            if (grab->surface()) {
+                Window *window = workspace()->findToplevel(
+                    [grab](const Window *w)->bool {
+                        if (!w->surface())
+                            return false;
+                        return grab->surface()->id() == w->surface()->id();
+                    }
+                );
+                if (window) {
+                    qWarning() << "found grab client, id:" << window->window() << "name:" << window->resourceName()
+                               << "class:" << window->resourceClass() << "captioin:" << window->caption()
+                               << "pid:" << window->pid();
+                }
+            }
         }
     );
     connect(m_xWaylandKeyboardGrabManager, &XWaylandKeyboardGrabManagerV1Interface::XWaylandKeyboardGrabV1Destroyed,
@@ -621,7 +637,7 @@ bool WaylandServer::init(InitializationFlags flags)
 
     m_clientManagement = new ClientManagementInterface(m_display, m_display);
     connect(m_clientManagement, &ClientManagementInterface::windowStatesRequest, this,
-        [this] () {
+        [] () {
             if (!workspace()) {
                 qWarning () << "windowStatesRequest before workspace initilized";
                 return;
@@ -630,7 +646,7 @@ bool WaylandServer::init(InitializationFlags flags)
         }
     );
     connect(m_clientManagement, &ClientManagementInterface::windowFromPointRequest, this,
-        [this] () {
+        [] () {
             if (!waylandServer())
                 return;
             auto client_management = waylandServer()->clientManagement();
@@ -642,17 +658,17 @@ bool WaylandServer::init(InitializationFlags flags)
         }
     );
     connect(m_clientManagement, &ClientManagementInterface::showSplitMenuRequest, this,
-        [this] (const QRect &botton_rect, uint32_t wid) {
+        [] (const QRect &botton_rect, uint32_t wid) {
             workspace()->showSplitMenu(botton_rect, wid);
         }
     );
     connect(m_clientManagement, &ClientManagementInterface::hideSplitMenuRequest, this,
-        [this] (bool delay) {
+        [] (bool delay) {
             workspace()->hideSplitMenu(delay);
         }
     );
     connect(m_clientManagement, &ClientManagementInterface::allWindowIdRequest, this,
-        [this] () {
+        [] () {
             if (!waylandServer())
                 return;
             auto client_management = waylandServer()->clientManagement();
@@ -666,7 +682,7 @@ bool WaylandServer::init(InitializationFlags flags)
         }
     );
     connect(m_clientManagement, &ClientManagementInterface::specificWindowStateRequest, this,
-        [this] (uint32_t wid) {
+        [] (uint32_t wid) {
             if (!waylandServer())
                 return;
             auto client_management = waylandServer()->clientManagement();
@@ -680,8 +696,14 @@ bool WaylandServer::init(InitializationFlags flags)
             if (!client) {
                 const WindowState invalid = {
                     .pid = 0,
-                    .windowId = 0
-                };
+                    .windowId = 0,
+                    .resourceName = {0},
+                    .geometry = {0, 0, 0, 0},
+                    .isMinimized = false,
+                    .isFullScreen = false,
+                    .isActive = false,
+                    .splitable = 0,
+                    .uuid = {0}};
                 client_management->sendSpecificWindowState(invalid);
                 return;
             }
@@ -692,13 +714,13 @@ bool WaylandServer::init(InitializationFlags flags)
             const std::string uuid = client->internalId().toString().toStdString();
             WindowState state = {
                 .pid = client->pid(),
-                .windowId = wid,
+                .windowId = static_cast<int32_t>(wid),
                 .resourceName = {0},
                 .geometry = {
-                    .x = client->frameGeometry().x(),
-                    .y = client->frameGeometry().y(),
-                    .width = client->frameGeometry().width(),
-                    .height = client->frameGeometry().height()
+                    .x = static_cast<int32_t>(client->frameGeometry().x()),
+                    .y = static_cast<int32_t>(client->frameGeometry().y()),
+                    .width = static_cast<int32_t>(client->frameGeometry().width()),
+                    .height = static_cast<int32_t>(client->frameGeometry().height())
                 },
                 .isMinimized = client->isMinimized(),
                 .isFullScreen = client->isFullScreen(),
@@ -770,6 +792,7 @@ bool WaylandServer::init(InitializationFlags flags)
     m_ddeRestrict = new DDERestrictInterface(m_display, m_display);
     m_ddeKvm = new DDEKvmInterface(m_display, m_display);
 
+    qCInfo(KWIN_CORE) << "Init wayland server finished, socket:" << socketName() << ", flags:" << flags;
     return true;
 }
 
@@ -928,6 +951,7 @@ int WaylandServer::createScreenLockerConnection()
     connect(m_screenLockerClientConnection, &KWaylandServer::ClientConnection::disconnected, this, [this]() {
         m_screenLockerClientConnection = nullptr;
     });
+    qCInfo(KWIN_CORE) << "ScreenLocker connection created, fd:" << socket.fd;
     return socket.fd;
 }
 
@@ -935,6 +959,7 @@ int WaylandServer::createXWaylandConnection()
 {
     const auto socket = createConnection();
     if (!socket.connection) {
+        qCWarning(KWIN_CORE) << "Failed to create XWayland connection.";
         return -1;
     }
     m_xwaylandConnection = socket.connection;
@@ -944,42 +969,51 @@ int WaylandServer::createXWaylandConnection()
         m_xwaylandConnection->setScaleOverride(kwinApp()->xwaylandScale());
     });
 
+    qCInfo(KWIN_CORE) << "XWayland connection created, fd:" << socket.fd;
     return socket.fd;
 }
 
 void WaylandServer::destroyXWaylandConnection()
 {
     if (!m_xwaylandConnection) {
+        qCWarning(KWIN_CORE) << "Destroy XWayland connection called but connection does not exist.";
         return;
     }
     m_xwaylandConnection->destroy();
     m_xwaylandConnection = nullptr;
+    qCInfo(KWIN_CORE) << "XWayland connection destroyed.";
 }
 
 int WaylandServer::createInputMethodConnection()
 {
     const auto socket = createConnection();
     if (!socket.connection) {
+        qCWarning(KWIN_CORE) << "Failed to create InputMethod connection.";
         return -1;
     }
     m_inputMethodServerConnection = socket.connection;
+    qCInfo(KWIN_CORE) << "InputMethod connection created, fd:" << socket.fd;
     return socket.fd;
 }
 
 void WaylandServer::destroyInputMethodConnection()
 {
     if (!m_inputMethodServerConnection) {
+        qCWarning(KWIN_CORE) << "Destroy InputMethod connection called but connection does not exist.";
         return;
     }
     m_inputMethodServerConnection->destroy();
     m_inputMethodServerConnection = nullptr;
+    qCInfo(KWIN_CORE) << "InputMethod connection destroyed.";
 }
 
 void WaylandServer::removeWindow(Window *c)
 {
+    qCInfo(KWIN_CORE) << "Removing window, window name:" << c->resourceName() << "window id:" << c->window() << "uuid:" << c->internalId().toByteArray();
     m_windows.removeAll(c);
     if (c->readyForPainting()) {
         Q_EMIT windowRemoved(c);
+        qCInfo(KWIN_CORE) << "Window ready for painting, removed. Window id:" << c->window();
     }
 }
 
