@@ -1198,7 +1198,7 @@ public:
 
     void move(const QPointF &point);
     void resize(const QSizeF &size);
-    void moveResize(const QRectF &rect);
+    void moveResize(const QRectF &rect, bool immediateResize = false);
     void moveResizeGeometry(const QRectF &rect);
 
     virtual QRectF resizeWithChecks(const QRectF &geometry, const QSizeF &s) = 0;
@@ -1656,7 +1656,6 @@ Q_SIGNALS:
     void clientMaximizedStateAboutToChange(KWin::Window *, MaximizeMode);
     void clientMaximizedStateChanged(KWin::Window *, MaximizeMode, bool animated = true);
     void clientMaximizedStateChanged(KWin::Window *c, bool h, bool v);
-    void clientMaximizedChanged(KWin::Window *, QRectF, QRectF, MaximizeMode);
     void transientChanged();
     void modalChanged();
     void quickTileModeChanged();
@@ -1857,7 +1856,7 @@ protected:
     };
     MoveResizeMode pendingMoveResizeMode() const;
     void setPendingMoveResizeMode(MoveResizeMode mode);
-    virtual void moveResizeInternal(const QRectF &rect, MoveResizeMode mode) = 0;
+    virtual void moveResizeInternal(const QRectF &rect, MoveResizeMode mode, bool immediateResize = false) = 0;
 
     /**
      * @returns whether the Window is currently in move resize mode
@@ -1972,7 +1971,7 @@ protected:
      * Default implementation does nothing.
      */
     virtual void doInteractiveResizeSync(const QRectF &rect);
-    void handleInteractiveMoveResize(int x, int y, int x_root, int y_root);
+    void handleInteractiveMoveResize(qreal x, qreal y, qreal x_root, qreal y_root);
     void handleInteractiveMoveResize(const QPointF &local, const QPointF &global);
     void dontInteractiveMoveResize();
 
