@@ -161,9 +161,6 @@ void XdgSurfaceV6Window::handleConfigureAcknowledged(quint32 serial)
 void XdgSurfaceV6Window::leaveInteractiveMoveResize()
 {
     Window::leaveInteractiveMoveResize();
-    if (m_plasmaShellSurface) {
-        m_plasmaShellSurface->resetPositionSet();
-    }
 }
 
 void XdgSurfaceV6Window::handleCommit()
@@ -307,7 +304,7 @@ void XdgSurfaceV6Window::resetHaveNextWindowGeometry()
     m_haveNextWindowGeometry = false;
 }
 
-void XdgSurfaceV6Window::moveResizeInternal(const QRectF &rect, MoveResizeMode mode)
+void XdgSurfaceV6Window::moveResizeInternal(const QRectF &rect, MoveResizeMode mode, bool immediateResize)
 {
     if (areGeometryUpdatesBlocked()) {
         setPendingMoveResizeMode(mode);
@@ -321,6 +318,9 @@ void XdgSurfaceV6Window::moveResizeInternal(const QRectF &rect, MoveResizeMode m
         if (requestedClientSize == clientSize()) {
             updateGeometry(rect);
         } else {
+            if (immediateResize) {
+                updateGeometry(rect);
+            }
             m_configureFlags |= XdgSurfaceV6Configure::ConfigurePosition;
             scheduleConfigure();
         }
@@ -374,9 +374,6 @@ void XdgSurfaceV6Window::destroyWindow()
 void XdgSurfaceV6Window::updateClientArea()
 {
     if (hasStrut()) {
-        if (isMaximized() && m_plasmaShellSurface) {
-            m_plasmaShellSurface->resetPositionSet();
-        }
         workspace()->updateClientArea();
     }
 }
@@ -509,6 +506,9 @@ void XdgSurfaceV6Window::installPlasmaShellSurface(PlasmaShellSurfaceInterface *
     m_plasmaShellSurface = shellSurface;
 
     auto updatePosition = [this, shellSurface] {
+        if (pos() == QPointF(shellSurface->position())) {
+            return;
+        }
         move(shellSurface->position());
     };
     auto moveUnderCursor = [this] {
@@ -2025,7 +2025,6 @@ void XdgToplevelV6Window::maximize(MaximizeMode mode, bool animated)
     if (m_requestedMaximizeMode == mode) {
         return;
     }
-    Q_EMIT clientMaximizedStateAboutToChange(this, mode);
     m_requestedMaximizeMode = mode;
 
     // call into decoration update borders
@@ -2107,15 +2106,12 @@ void XdgToplevelV6Window::maximize(MaximizeMode mode, bool animated)
         } else {
             updateQuickTileMode(QuickTileFlag::None);
         }
-        // When maximizing the window, it is necessary to reset the positionset.
-        if (m_plasmaShellSurface) {
-            m_plasmaShellSurface->resetPositionSet();
-        }
     } else {
         updateQuickTileMode(QuickTileFlag::None);
     }
 
     moveResize(geometry);
+    Q_EMIT clientMaximizedStateAboutToChange(this, mode);
 
     if (oldQuickTileMode != quickTileMode()) {
         doSetQuickTileMode();
