@@ -35,7 +35,6 @@
 #include "workspace.h"
 #include "useractions.h"
 #include "wayland/dderestrict_interface.h"
-#include "scene/surfaceitem.h"
 
 #include <KDecoration2/DecoratedClient>
 #include <KDecoration2/Decoration>
@@ -162,9 +161,6 @@ void XdgSurfaceWindow::handleConfigureAcknowledged(quint32 serial)
 void XdgSurfaceWindow::leaveInteractiveMoveResize()
 {
     Window::leaveInteractiveMoveResize();
-    if (m_plasmaShellSurface) {
-        m_plasmaShellSurface->resetPositionSet();
-    }
 }
 
 void XdgSurfaceWindow::handleCommit()
@@ -257,8 +253,6 @@ static QRectF gravitateGeometry(const QRectF &rect, const QRectF &bounds, Gravit
 
 void XdgSurfaceWindow::handleNextWindowGeometry()
 {
-    const QRectF boundingGeometry = surface()->boundingRect();
-
     // The effective window geometry is defined as the intersection of the window geometry
     // and the rectangle that bounds the main surface and all of its sub-surfaces. If the
     // client hasn't specified the window geometry, we must fallback to the bounding geometry.
@@ -266,10 +260,13 @@ void XdgSurfaceWindow::handleNextWindowGeometry()
     // window geometry.
 
     m_windowGeometry = m_shellSurface->windowGeometry();
-    if (m_windowGeometry.isValid()) {
-        m_windowGeometry &= boundingGeometry;
-    } else {
-        m_windowGeometry = boundingGeometry;
+    if (surface()->viewportExtension()) {
+        const QRectF boundingGeometry = surface()->boundingRect();
+        if (m_windowGeometry.isValid()) {
+            m_windowGeometry &= boundingGeometry;
+        } else {
+            m_windowGeometry = boundingGeometry;
+        }
     }
 
     if (m_windowGeometry.isEmpty()) {
@@ -310,7 +307,7 @@ void XdgSurfaceWindow::resetHaveNextWindowGeometry()
     m_haveNextWindowGeometry = false;
 }
 
-void XdgSurfaceWindow::moveResizeInternal(const QRectF &rect, MoveResizeMode mode)
+void XdgSurfaceWindow::moveResizeInternal(const QRectF &rect, MoveResizeMode mode, bool immediateResize)
 {
     if (areGeometryUpdatesBlocked()) {
         setPendingMoveResizeMode(mode);
@@ -324,6 +321,9 @@ void XdgSurfaceWindow::moveResizeInternal(const QRectF &rect, MoveResizeMode mod
         if (requestedClientSize == clientSize()) {
             updateGeometry(rect);
         } else {
+            if (immediateResize) {
+                updateGeometry(rect);
+            }
             m_configureFlags |= XdgSurfaceConfigure::ConfigurePosition;
             scheduleConfigure();
         }
@@ -334,8 +334,8 @@ void XdgSurfaceWindow::moveResizeInternal(const QRectF &rect, MoveResizeMode mod
         }
         m_configureFlags.setFlag(XdgSurfaceConfigure::ConfigurePosition, false);
         if (size() != rect.size()) {
-            if (auto toplevelwindow = qobject_cast<const XdgToplevelWindow *>(this)) {
-                return ;
+            if (qobject_cast<const XdgToplevelWindow *>(this)) {
+                return;
             }
         }
         updateGeometry(QRectF(rect.topLeft(), size()));
@@ -351,7 +351,6 @@ QRectF XdgSurfaceWindow::frameRectToBufferRect(const QRectF &rect) const
 
 void XdgSurfaceWindow::destroyWindow()
 {
-    FUNC_DEBUG_LOG(Q_FUNC_INFO, window());
     markAsZombie();
     if (isInteractiveMoveResize()) {
         leaveInteractiveMoveResize();
@@ -387,9 +386,6 @@ void XdgSurfaceWindow::destroyWindow()
 void XdgSurfaceWindow::updateClientArea()
 {
     if (hasStrut()) {
-        if (isMaximized() && m_plasmaShellSurface) {
-            m_plasmaShellSurface->resetPositionSet();
-        }
         workspace()->updateClientArea();
     }
 }
@@ -522,6 +518,9 @@ void XdgSurfaceWindow::installPlasmaShellSurface(PlasmaShellSurfaceInterface *sh
     m_plasmaShellSurface = shellSurface;
 
     auto updatePosition = [this, shellSurface] {
+        if (pos() == QPointF(shellSurface->position())) {
+            return;
+        }
         move(shellSurface->position());
     };
     auto moveUnderCursor = [this] {
@@ -1801,8 +1800,6 @@ void XdgToplevelWindow::updateMaximizeMode(MaximizeMode maximizeMode)
     setMaximized(maximizeMode & MaximizeHorizontal && maximizeMode & MaximizeVertical);
     Q_EMIT clientMaximizedStateChanged(this, maximizeMode);
     Q_EMIT clientMaximizedStateChanged(this, maximizeMode & MaximizeHorizontal, maximizeMode & MaximizeVertical);
-    if (maximizeMode == MaximizeFull || maximizeMode == MaximizeRestore)
-        Q_EMIT clientMaximizedChanged(this, m_restoreMaxiArea, frameGeometry(), maximizeMode);
 }
 
 void XdgToplevelWindow::updateFullScreenMode(bool set)
@@ -2042,11 +2039,6 @@ void XdgToplevelWindow::maximize(MaximizeMode mode, bool animated)
     if (m_requestedMaximizeMode == mode) {
         return;
     }
-    if (auto item = surfaceItem()) {
-        item->discardPixmap();
-    }
-    m_restoreMaxiArea = frameGeometry();
-    Q_EMIT clientMaximizedStateAboutToChange(this, mode);
     m_requestedMaximizeMode = mode;
 
     // call into decoration update borders
@@ -2128,16 +2120,13 @@ void XdgToplevelWindow::maximize(MaximizeMode mode, bool animated)
         } else {
             updateQuickTileMode(QuickTileFlag::None);
         }
-        // When maximizing the window, it is necessary to reset the positionset.
-        if (m_plasmaShellSurface) {
-            m_plasmaShellSurface->resetPositionSet();
-        }
     } else {
         updateQuickTileMode(QuickTileFlag::None);
     }
 
     qCDebug(KWIN_CORE) << "set window max mode: " << m_requestedMaximizeMode << window();
     moveResize(geometry);
+    Q_EMIT clientMaximizedStateAboutToChange(this, mode);
 
     if (oldQuickTileMode != quickTileMode()) {
         doSetQuickTileMode();
@@ -2243,9 +2232,6 @@ QRectF XdgPopupWindow::transientPlacement(const QRectF &bounds) const
     const XdgPositioner positioner = m_shellSurface->positioner();
 
     const QSize desiredSize = positioner.size();
-    if (m_plasmaShellSurface && m_plasmaShellSurface->isPositionSet()) {
-        return QRectF(m_plasmaShellSurface->position(), desiredSize);
-    }
 
     const QPointF parentPosition = transientFor()->framePosToClientPos(transientFor()->pos());
 
